<?php

namespace Drupal\me_config_pages\Controller;

use Drupal\config_pages\ConfigPagesTypeInterface;
use Drupal\config_pages\Controller\ConfigPagesController;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Controller override for Config Pages form.
 */
class MercuryEditorConfigPagesController extends ConfigPagesController {

  /**
   * {@inheritDoc}
   */
  public function classInit(ConfigPagesTypeInterface $config_pages_type = NULL) {
    // These services should probably be injected somehow.
    /** @var \Drupal\Core\Routing\RouteMatchInterface */
    $route_match = \Drupal::service('current_route_match');
    // Using the loader service over type manager to be
    // able to pass context data in.
    /** @var \Drupal\config_pages\ConfigPagesLoaderServiceInterface */
    $cp_loader = \Drupal::service('config_pages.loader');
    /** @var \Drupal\mercury_editor\MercuryEditorTempstore */
    $me_tempstore = \Drupal::service('mercury_editor.tempstore_repository');

    if ($route_match->getRouteName() !== 'mercury_editor.editor') {
      $cp_type = $config_pages_type->id();
      /** @var \Drupal\config_pages\ConfigPagesTypeInterface */
      $cp_type_entity = $this->configPagesTypeStorage->load($cp_type);

      if (empty($cp_type_entity)) {
        throw new NotFoundHttpException();
      }

      $context_data = $cp_type_entity->getContextData();
      $config_page = $cp_loader->load($cp_type, $context_data);

      if (is_null($config_page)) {
        $config_page = $this->configPagesStorage->create([
          'type' => $cp_type,
        ]);
      }

      $me_tempstore->set($config_page);
      $url = Url::fromRoute('mercury_editor.editor', [
        'mercury_editor_entity' => $config_page->uuid(),
      ]);

      return new RedirectResponse($url->toString());
    }

    return parent::classInit($config_pages_type);
  }

}
