<?php

declare(strict_types=1);

namespace Drupal\me_config_pages\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * ME Config Pages settings form.
 */
class MercuryEditorConfigPagesSettingsForm extends ConfigFormBase {

  /**
   * SettingsForm constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager service.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    protected EntityTypeManagerInterface $entityTypeManager
  ) {
    parent::__construct($config_factory);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) : static {
    return new static(
      $container->get('config.factory'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() : string {
    return 'me_config_pages_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() : array {
    return [
      'me_config_pages.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) : array {
    $config = $this->configFactory()->getEditable('me_config_pages.settings');

    $form['config_pages_types'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Config Pages Types'),
      '#description' => $this->t('Select which Config Pages should support Mercury Editor.'),
      '#options' => $this->getConfigPagesOptions(),
      '#default_value' => $config->get('config_pages_types') ?? [],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->configFactory()->getEditable('me_config_pages.settings');
    $config->set('config_pages_types', $form_state->getValue('config_pages_types'));
    $config->save();

    $this->messenger()->addMessage($this->t('Updated ME Config Pages settings.'));
  }

  /**
   * Get an options list of Config Pages for forms.
   *
   * @return array
   *   The values for a form.
   */
  protected function getConfigPagesOptions() : array {
    $config_page_type_storage = $this->entityTypeManager->getStorage('config_pages_type');
    $types = [];

    /** @var \Drupal\config_pages\ConfigPagesTypeInterface $bundle */
    foreach ($config_page_type_storage->loadMultiple() as $machine_name => $bundle) {
      $types[$machine_name] = $bundle->label();
    }

    return $types;
  }

}
